@@ -3,6 +3,7 @@
 // file, You can obtain one at https://mozilla.org/MPL/2.0/.
 
 use crate::error::ParseError;
+use std::collections::HashMap;
 use std::result::Result;
 use std::time::Duration;
 
@@ -12,6 +13,9 @@ use std::time::Duration;
 pub fn parse<R: std::io::Read>(side_file: R) -> Result<File, ParseError> {
     let side: format::SideFile =
         serde_json::from_reader(side_file).map_err(ParseError::FormatError)?;
+
+    file_has_uniq_tests(&side)?;
+
     let mut tests = Vec::new();
     for test in side.tests {
         let mut commands = Vec::with_capacity(test.commands.len());
@@ -40,6 +44,18 @@ pub fn parse<R: std::io::Read>(side_file: R) -> Result<File, ParseError> {
     })
 }
 
+fn file_has_uniq_tests(file: &format::SideFile) -> Result<(), ParseError> {
+    let mut seen = HashMap::new();
+    for (i, test) in file.tests.iter().enumerate() {
+        let old = seen.insert(test.name.clone(), i);
+        if let Some(index) = old {
+            return Err(ParseError::NotUniqTestName(index, i));
+        }
+    }
+
+    Ok(())
+}
+
 fn parse_cmd(command: &format::Command) -> Result<Cmd, ParseError> {
     let parse_fn = match command.cmd.as_str() {
         "open" => Cmd::parse_open,
@@ -48,9 +64,11 @@ fn parse_cmd(command: &format::Command) -> Result<Cmd, ParseError> {
         "executeScript" => Cmd::parse_execute_script,
         "executeScriptAsync" => Cmd::parse_execute_async_script,
         "waitForElementVisible" => Cmd::parse_wait_for_visible,
+        "waitForElementNotVisible" => Cmd::parse_wait_for_not_visible,
         "waitForElementEditable" => Cmd::parse_wait_for_editable,
-        "waitForElementNotPresent" => Cmd::parse_wait_for_not_present,
+        "waitForElementNotEditable" => Cmd::parse_wait_for_not_editable,
         "waitForElementPresent" => Cmd::parse_wait_for_present,
+        "waitForElementNotPresent" => Cmd::parse_wait_for_not_present,
         "select" => Cmd::parse_select,
         "addSelection" => Cmd::parse_add_selection,
         "echo" => Cmd::parse_echo,
@@ -75,6 +93,7 @@ fn parse_cmd(command: &format::Command) -> Result<Cmd, ParseError> {
         "assertNotChecked" => Cmd::parse_assert_not_checked,
         "assertPrompt" => Cmd::parse_assert_prompt,
         "assertSelectedValue" => Cmd::parse_assert_selected_value,
+        "assertNotSelectedValue" => Cmd::parse_assert_not_selected_value,
         "assertText" => Cmd::parse_assert_text,
         "assertNotText" => Cmd::parse_assert_not_text,
         "doubleClick" => Cmd::parse_double_click,
@@ -82,8 +101,30 @@ fn parse_cmd(command: &format::Command) -> Result<Cmd, ParseError> {
         "sendKeys" => Cmd::parse_send_keys,
         "type" => Cmd::parse_type,
         "check" => Cmd::parse_check,
+        "uncheck" => Cmd::parse_uncheck,
         "mouseUp" => Cmd::parse_mouse_up,
         "mouseDown" => Cmd::parse_mouse_down,
+        "chooseCancelOnNextConfirmation" => Cmd::parse_choose_cancel_on_next_confirmation,
+        "chooseOkOnNextConfirmation" => Cmd::parse_choose_ok_on_next_confirmation,
+        "chooseCancelOnNextPrompt" => Cmd::parse_choose_cancel_on_next_prompt,
+        "assertTitle" => Cmd::parse_assert_title,
+        "storeTitle" => Cmd::parse_store_title,
+        "assertValue" => Cmd::parse_assert_value,
+        "assertConfirmation" => Cmd::parse_assert_confirmation,
+        "assertSelectedLabel" => Cmd::parse_assert_selected_label,
+        "times" => Cmd::parse_times,
+        "run" => Cmd::parse_run,
+        "storeValue" => Cmd::parse_store_value,
+        "storeAttribute" => Cmd::parse_store_attribute,
+        "storeJson" => Cmd::parse_store_json,
+        "removeSelection" => Cmd::parse_remove_selection,
+        "assertElementPresent" => Cmd::parse_assert_element_present,
+        "assertElementNotPresent" => Cmd::parse_assert_element_not_present,
+        "assertElementEditable" => Cmd::parse_assert_element_editable,
+        "assertElementNotEditable" => Cmd::parse_assert_element_not_editable,
+        // It seems like possible 2 names may be used [assertEditable|assertElementEditable]
+        "assertEditable" => Cmd::parse_assert_element_editable,
+        "assertNotEditable" => Cmd::parse_assert_element_not_editable,
         cmd if cmd.is_empty() || cmd.starts_with("//") => {
             // We create an empty command to not lose an order of commands.
             // It's usefull for error messages to not break the indexes of commands from a file.
@@ -184,15 +225,21 @@ pub enum Cmd {
         target: Target,
         locator: SelectLocator,
     },
+    AddSelection(Target, String),
+    RemoveSelection(Target, String),
     WaitForElementVisible {
         target: Target,
         timeout: Duration,
     },
+    WaitForElementNotVisible {
+        target: Target,
+        timeout: Duration,
+    },
     WaitForElementEditable {
         target: Target,
         timeout: Duration,
     },
-    WaitForElementNotPresent {
+    WaitForElementNotEditable {
         target: Target,
         timeout: Duration,
     },
@@ -200,6 +247,10 @@ pub enum Cmd {
         target: Target,
         timeout: Duration,
     },
+    WaitForElementNotPresent {
+        target: Target,
+        timeout: Duration,
+    },
     StoreText {
         var: String,
         target: Target,
@@ -252,6 +303,7 @@ pub enum Cmd {
     AssertChecked(Target),
     AssertNotChecked(Target),
     AssertSelectedValue(Target, String),
+    AssertNotSelectedValue(Target, String),
     AssertText(Target, String),
     AssertNotText(Target, String),
     DoubleClick(Target),
@@ -259,8 +311,26 @@ pub enum Cmd {
     SendKeys(Target, String),
     Type(Target, String),
     Check(Target),
+    UnCheck(Target),
     MouseUp(Target),
     MouseDown(Target),
+    ChooseCancelOnNextConfirmation,
+    ChooseOkOnNextConfirmation,
+    ChooseCancelOnNextPrompt,
+    AssertTitle(String),
+    StoreTitle(String),
+    AssertValue(Target, String),
+    AssertConfirmation(String),
+    AssertSelectedLabel(Target, String),
+    Times(String),
+    RunTest(String),
+    StoreJson(String, String),
+    StoreValue(Target, String),
+    StoreAttribute(Target, String, String),
+    AssertElementPresent(Target),
+    AssertElementNotPresent(Target),
+    AssertEditable(Target),
+    AssertNotEditable(Target),
 }
 
 impl Cmd {
@@ -311,6 +381,14 @@ impl Cmd {
         Ok(Self::WaitForElementVisible { target, timeout })
     }
 
+    fn parse_wait_for_not_visible(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+        let timeout = cast_timeout(&c.value)?;
+
+        Ok(Self::WaitForElementNotVisible { target, timeout })
+    }
+
     fn parse_wait_for_editable(c: &format::Command) -> Result<Self, ParseError> {
         let location = parse_location(&c.target)?;
         let target = Target::new(location);
@@ -319,6 +397,14 @@ impl Cmd {
         Ok(Self::WaitForElementEditable { target, timeout })
     }
 
+    fn parse_wait_for_not_editable(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+        let timeout = cast_timeout(&c.value)?;
+
+        Ok(Self::WaitForElementNotEditable { target, timeout })
+    }
+
     fn parse_wait_for_not_present(c: &format::Command) -> Result<Self, ParseError> {
         let location = parse_location(&c.target)?;
         let target = Target::new(location);
@@ -344,11 +430,17 @@ impl Cmd {
     }
 
     fn parse_add_selection(c: &format::Command) -> Result<Self, ParseError> {
-        let locator = SelectLocator::Label(c.value.clone());
         let location = parse_location(&c.target)?;
         let target = Target::new(location);
 
-        Ok(Self::Select { target, locator })
+        Ok(Self::AddSelection(target, c.value.clone()))
+    }
+
+    fn parse_remove_selection(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+
+        Ok(Self::RemoveSelection(target, c.value.clone()))
     }
 
     fn parse_echo(c: &format::Command) -> Result<Self, ParseError> {
@@ -510,6 +602,12 @@ impl Cmd {
         Ok(Self::AssertSelectedValue(target, c.value.clone()))
     }
 
+    fn parse_assert_not_selected_value(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+        Ok(Self::AssertNotSelectedValue(target, c.value.clone()))
+    }
+
     fn parse_assert_text(c: &format::Command) -> Result<Self, ParseError> {
         let location = parse_location(&c.target)?;
         let target = Target::new(location);
@@ -552,6 +650,12 @@ impl Cmd {
         Ok(Self::Check(target))
     }
 
+    fn parse_uncheck(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+        Ok(Self::UnCheck(target))
+    }
+
     fn parse_mouse_down(c: &format::Command) -> Result<Self, ParseError> {
         let location = parse_location(&c.target)?;
         let target = Target::new(location);
@@ -563,6 +667,99 @@ impl Cmd {
         let target = Target::new(location);
         Ok(Self::MouseUp(target))
     }
+
+    fn parse_choose_cancel_on_next_confirmation(_: &format::Command) -> Result<Self, ParseError> {
+        Ok(Self::ChooseCancelOnNextConfirmation)
+    }
+
+    fn parse_choose_ok_on_next_confirmation(_: &format::Command) -> Result<Self, ParseError> {
+        Ok(Self::ChooseOkOnNextConfirmation)
+    }
+
+    fn parse_choose_cancel_on_next_prompt(_: &format::Command) -> Result<Self, ParseError> {
+        Ok(Self::ChooseCancelOnNextPrompt)
+    }
+
+    fn parse_assert_title(c: &format::Command) -> Result<Self, ParseError> {
+        Ok(Self::AssertTitle(c.target.clone()))
+    }
+
+    fn parse_store_title(c: &format::Command) -> Result<Self, ParseError> {
+        Ok(Self::StoreTitle(c.value.clone()))
+    }
+
+    fn parse_assert_value(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+        Ok(Self::AssertValue(target, c.value.clone()))
+    }
+
+    fn parse_assert_confirmation(c: &format::Command) -> Result<Self, ParseError> {
+        let expected = c.target.clone();
+        Ok(Self::AssertConfirmation(expected))
+    }
+
+    fn parse_assert_selected_label(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+        Ok(Self::AssertSelectedLabel(target, c.value.clone()))
+    }
+
+    fn parse_times(c: &format::Command) -> Result<Self, ParseError> {
+        Ok(Self::Times(c.target.clone()))
+    }
+
+    fn parse_run(c: &format::Command) -> Result<Self, ParseError> {
+        Ok(Self::RunTest(c.target.clone()))
+    }
+
+    fn parse_store_value(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+        Ok(Self::StoreValue(target, c.value.clone()))
+    }
+
+    fn parse_store_json(c: &format::Command) -> Result<Self, ParseError> {
+        Ok(Self::StoreJson(c.target.clone(), c.value.clone()))
+    }
+
+    fn parse_store_attribute(c: &format::Command) -> Result<Self, ParseError> {
+        let (target, attribute) = c.target.split_once('@').ok_or_else(|| {
+            ParseError::LocatorFormatError("a attribute splitter '@' wasn't found".to_string())
+        })?;
+
+        let location = parse_location(target)?;
+        let target = Target::new(location);
+        Ok(Self::StoreAttribute(
+            target,
+            attribute.to_string(),
+            c.value.clone(),
+        ))
+    }
+
+    fn parse_assert_element_present(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+        Ok(Self::AssertElementPresent(target))
+    }
+
+    fn parse_assert_element_not_present(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+        Ok(Self::AssertElementNotPresent(target))
+    }
+
+    fn parse_assert_element_editable(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+        Ok(Self::AssertEditable(target))
+    }
+
+    fn parse_assert_element_not_editable(c: &format::Command) -> Result<Self, ParseError> {
+        let location = parse_location(&c.target)?;
+        let target = Target::new(location);
+        Ok(Self::AssertNotEditable(target))
+    }
 }
 
 /// Target represents a locator of html element
@@ -836,6 +1033,44 @@ mod tests {
         }
     }
 
+    #[test]
+    fn _error_on_tests_with_not_uniq_names() {
+        let file: Vec<u8> = r#"{
+            "id": "bfc1bd56-39bd-4a0d-be2b-583ad75ac104",
+            "version": "2.0",
+            "name": "",
+            "url": "",
+            "tests": [
+                {
+                    "id": "5d61ce01-d373-4b14-a1a1-7474a4e192e5",
+                    "name": "basic",
+                    "commands": []
+                },
+                {
+                    "id": "5d61ce01-d373-4b14-a1a1-7474a4e192e5",
+                    "name": "basic",
+                    "commands": []
+                }
+            ],
+            "suites": [{
+              "id": "925de5ce-03ae-4dcb-9146-c956ff3f090d",
+              "name": "Default Suite",
+              "persistSession": false,
+              "parallel": false,
+              "timeout": 300,
+              "tests": ["5d61ce01-d373-4b14-a1a1-7474a4e192e5"]
+            }],
+            "urls": [""],
+            "plugins": []
+          }"#
+        .as_bytes()
+        .to_vec();
+
+        let reader = file.as_slice();
+        let err = parse(reader).unwrap_err();
+        assert!(matches!(err, ParseError::NotUniqTestName(0, 1)))
+    }
+
     fn side_file() -> Vec<u8> {
         r#"{
             "id": "bfc1bd56-39bd-4a0d-be2b-583ad75ac104",
