@@ -1,6 +1,4 @@
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at https://mozilla.org/MPL/2.0/.
+#![cfg(test)]
 
 use crate::{
     parser::{Cmd, Command},
@@ -622,6 +620,49 @@ mod flow {
         assert_eq!(echo_vector.lock().unwrap()[2], "E3");
     }
 
+    #[tokio::test]
+    async fn test_times() {
+        let file = blank_file(vec![
+            Cmd::Open("http://example.com".to_string()),
+            Cmd::Times("4".to_string()),
+            Cmd::Echo("".to_string()),
+            Cmd::End,
+        ]);
+        let client = Client::new();
+        let mut runner = Runner::_new(client.clone());
+
+        let echo_counter: Arc<Mutex<usize>> = Arc::default();
+        let echo_counter1 = echo_counter.clone();
+        runner.set_echo(move |_| *echo_counter1.lock().unwrap() += 1);
+
+        let res = runner.run(&file).await;
+        assert!(res.is_ok());
+
+        assert_eq!(*echo_counter.lock().unwrap(), 4);
+    }
+
+    #[tokio::test]
+    async fn test_times_with_var() {
+        let file = blank_file(vec![
+            Cmd::Open("http://example.com".to_string()),
+            Cmd::Times("${N}".to_string()),
+            Cmd::Echo("".to_string()),
+            Cmd::End,
+        ]);
+        let client = Client::new();
+        let mut runner = Runner::_new(client.clone());
+        runner.save_value("N".to_string(), 4u64.into());
+
+        let echo_counter: Arc<Mutex<usize>> = Arc::default();
+        let echo_counter1 = echo_counter.clone();
+        runner.set_echo(move |_| *echo_counter1.lock().unwrap() += 1);
+
+        let res = runner.run(&file).await;
+        assert!(res.is_ok());
+
+        assert_eq!(*echo_counter.lock().unwrap(), 4);
+    }
+
     #[tokio::test]
     async fn test_open_relative_url() {
         let file = File::new(
@@ -779,6 +820,15 @@ mod flow {
                 Ok(())
             }
 
+            async fn wait_for_not_visible(
+                &mut self,
+                _: Locator,
+                _: Duration,
+            ) -> Result<(), RunnerErrorKind> {
+                self.inc(Call::W8NotVisib);
+                Ok(())
+            }
+
             async fn wait_for_not_present(
                 &mut self,
                 _: Locator,
@@ -806,6 +856,15 @@ mod flow {
                 Ok(())
             }
 
+            async fn wait_for_not_editable(
+                &mut self,
+                _: Locator,
+                _: Duration,
+            ) -> Result<(), RunnerErrorKind> {
+                self.inc(Call::W8NotEdit);
+                Ok(())
+            }
+
             async fn current_url(&mut self) -> Result<url::Url, RunnerErrorKind> {
                 self.inc(Call::CurrentUrl);
                 Ok(url::Url::parse("http://example.com").unwrap())
@@ -864,6 +923,11 @@ mod flow {
                 self.inc(Call::MouseUp);
                 Ok(())
             }
+
+            async fn title(&mut self) -> Result<String, RunnerErrorKind> {
+                self.inc(Call::Title);
+                Ok(String::new())
+            }
         }
 
         pub struct Element(Arc<Client>);
@@ -925,40 +989,34 @@ mod flow {
                 self.inc(Call::SendKeys);
                 Ok(())
             }
+
+            async fn select_by_label(mut self, _: &str) -> Result<Self::Driver, RunnerErrorKind> {
+                self.inc(Call::SelectByLabel);
+                Ok(self.0)
+            }
+
+            async fn is_selected(&mut self) -> Result<bool, RunnerErrorKind> {
+                self.inc(Call::IsSelected);
+                Ok(true)
+            }
+
+            async fn is_present(&mut self) -> Result<bool, RunnerErrorKind> {
+                self.inc(Call::IsPresent);
+                Ok(true)
+            }
+
+            async fn is_enabled(&mut self) -> Result<bool, RunnerErrorKind> {
+                self.inc(Call::IsEnabled);
+                Ok(true)
+            }
         }
 
         #[derive(Clone, Default)]
         pub struct CallCount {
-            open: usize,
-            click: usize,
-            find: usize,
-            findall: usize,
-            goto: usize,
-            exec: usize,
-            exec_async: usize,
-            close: usize,
-            current_url: usize,
-            set_w_size: usize,
-            w_8_visib: usize,
-            w_8_pres: usize,
-            w_8_npres: usize,
-            w_8_edit: usize,
-            attr: usize,
-            prop: usize,
-            text: usize,
-            html: usize,
-            select_by_index: usize,
-            select_by_value: usize,
-            alert_text: usize,
-            alert_accept: usize,
-            alert_dissmis: usize,
-            double_click: usize,
-            send_keys: usize,
-            mouse_up: usize,
-            mouse_down: usize,
+            inner: std::collections::HashMap<Call, usize>,
         }
 
-        #[derive(Hash, PartialEq, Eq)]
+        #[derive(Hash, PartialEq, Eq, Clone)]
         pub enum Call {
             Open,
             Click,
@@ -971,15 +1029,18 @@ mod flow {
             CurrentUrl,
             SetWSize,
             W8Visib,
+            W8NotVisib,
             W8Pres,
             W8NPres,
             W8Edit,
+            W8NotEdit,
             Attr,
             Prop,
             Text,
             Html,
             SelectByIndex,
             SelectByValue,
+            SelectByLabel,
             AlertText,
             AlertAccept,
             AlertDissmis,
@@ -987,75 +1048,23 @@ mod flow {
             SendKeys,
             MouseDown,
             MouseUp,
+            Title,
+            IsSelected,
+            IsPresent,
+            IsEnabled,
         }
 
         impl Index<Call> for CallCount {
             type Output = usize;
 
-            fn index(&self, count: Call) -> &Self::Output {
-                match count {
-                    Call::Open => &self.open,
-                    Call::Click => &self.click,
-                    Call::Find => &self.find,
-                    Call::FindAll => &self.findall,
-                    Call::Goto => &self.goto,
-                    Call::Exec => &self.exec,
-                    Call::ExecAsync => &self.exec_async,
-                    Call::Close => &self.close,
-                    Call::CurrentUrl => &self.current_url,
-                    Call::SetWSize => &self.set_w_size,
-                    Call::W8Visib => &self.w_8_visib,
-                    Call::W8Pres => &self.w_8_pres,
-                    Call::W8NPres => &self.w_8_npres,
-                    Call::W8Edit => &self.w_8_edit,
-                    Call::Attr => &self.attr,
-                    Call::Prop => &self.prop,
-                    Call::Text => &self.text,
-                    Call::Html => &self.html,
-                    Call::SelectByIndex => &self.select_by_index,
-                    Call::SelectByValue => &self.select_by_value,
-                    Call::AlertText => &self.alert_text,
-                    Call::AlertAccept => &self.alert_accept,
-                    Call::AlertDissmis => &self.alert_dissmis,
-                    Call::DoubleClick => &self.double_click,
-                    Call::SendKeys => &self.send_keys,
-                    Call::MouseDown => &self.mouse_down,
-                    Call::MouseUp => &self.mouse_up,
-                }
+            fn index(&self, call: Call) -> &Self::Output {
+                self.inner.get(&call).unwrap()
             }
         }
 
         impl IndexMut<Call> for CallCount {
-            fn index_mut(&mut self, count: Call) -> &mut Self::Output {
-                match count {
-                    Call::Open => &mut self.open,
-                    Call::Click => &mut self.click,
-                    Call::Find => &mut self.find,
-                    Call::FindAll => &mut self.findall,
-                    Call::Goto => &mut self.goto,
-                    Call::Exec => &mut self.exec,
-                    Call::ExecAsync => &mut self.exec_async,
-                    Call::Close => &mut self.close,
-                    Call::CurrentUrl => &mut self.current_url,
-                    Call::SetWSize => &mut self.set_w_size,
-                    Call::W8Visib => &mut self.w_8_visib,
-                    Call::W8Pres => &mut self.w_8_pres,
-                    Call::W8NPres => &mut self.w_8_npres,
-                    Call::W8Edit => &mut self.w_8_edit,
-                    Call::Attr => &mut self.attr,
-                    Call::Prop => &mut self.prop,
-                    Call::Text => &mut self.text,
-                    Call::Html => &mut self.html,
-                    Call::SelectByIndex => &mut self.select_by_index,
-                    Call::SelectByValue => &mut self.select_by_value,
-                    Call::AlertText => &mut self.alert_text,
-                    Call::AlertAccept => &mut self.alert_accept,
-                    Call::AlertDissmis => &mut self.alert_dissmis,
-                    Call::DoubleClick => &mut self.double_click,
-                    Call::SendKeys => &mut self.send_keys,
-                    Call::MouseDown => &mut self.mouse_down,
-                    Call::MouseUp => &mut self.mouse_up,
-                }
+            fn index_mut(&mut self, call: Call) -> &mut Self::Output {
+                self.inner.entry(call).or_default()
             }
         }
     }
